<?php
/**  
* Plugin Name: Handprint  
* Plugin URI: https://wordpress.org/plugins/handprint
* Description: Make your online checkout planet positive with Handprint
* Author: Handprint Tech Pte Ltd
* Version: 1.3.3.2
* Author URI: https://handprint.tech
* Developer: Manish Gupta
* Developer URI: https://dashboard.handprint.tech
* Requires PHP: 7.0
* Text Domain: woocommerce-extension
* Domain Path: /languages/
* License: GPL3+ 
* License URI: https://www.gnu.org/licenses/gpl-3.0.txt
*/
include(plugin_dir_path(  __FILE__ ).'includes/admin/config/loadcredential.php');
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

load_plugin_textdomain('handprint',false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {

    register_activation_hook( __FILE__, function() {
        add_option( 'Activated_Plugin', 'handprint' );
      /* activation code here */
    });

    if ( ! class_exists( 'WC_handprint' ) ) :
        class WC_handprint {

          /**
          * Construct the plugin. 
          */
          public function __construct() {
        
            add_action( 'plugins_loaded', array( $this, 'init' ) );
          }

          /**
          * Initialize the plugin.
          */
          public function init() {
            // Checks if WooCommerce is installed.
            if ( class_exists( 'WC_Integration' ) ) {
              // Include our integration class.
              include_once 'includes/admin/class-wc-admin-saas-integration.php';
              // Register the integration.
              add_filter( 'woocommerce_integrations', array( $this, 'add_integration' ) );

              // Set the plugin slug
              define( 'MY_PLUGIN_SLUG', 'wc-settings' );

              // Setting action for plugin
              add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'WC_handprint_action_links' );
            }

            if ( is_admin() ) {
                add_action( 'admin_menu', 'add_saas_menu_entry', 100 );
            } 

            function add_saas_menu_entry() {
                add_submenu_page(
                    'woocommerce',
                    __( 'Handprint','handprint'),
                    __( 'Handprint','handprint' ),
                    'manage_woocommerce', // Required user capability
                    menu_page_url( MY_PLUGIN_SLUG, false ) .'&tab=integration&section=saas_pledge',
                    '', 7
                );
            }

            add_action( 'admin_init', 'load_plugin' );
            function load_plugin() {
                if ( is_admin() && ( get_option('Activated_Plugin') == 'handprint' ) ) {
                    delete_option('Activated_Plugin');
                    /* do some stuff once right after activation */
                    
                    wp_redirect( APIURL . '/ext/api/v2/congratulations?type=woocommerce' );
                }
            }

            #############Display Pledge section#############
            add_action( 'woocommerce_after_cart_table', 'custom_after_cart_table' );
            ################################################

            #############Fires after WordPress has finished loading, but before any headers are sent#############
            add_action( 'init', 'script_enqueuer' );
            #####################################################################################################

            #############Define the action for the hook created, for Logged-In Users#############
            add_action( 'wp_ajax_set_hp_session', 'set_hp_session' );
            #####################################################################################

            #############Define the action for the hook created, for Logged-Out Users#############
            add_action( 'wp_ajax_nopriv_set_hp_session', 'set_hp_session' );
            ######################################################################################

            ##########When New Order Created##########
            add_action( 'woocommerce_new_order', 'save_order_to_saas',  1, 1  );
            ##########################################

            ##############Add Footer Script##############
            add_action( 'wp_footer', 'footer_script' );
            #############################################

            #########Save custom field value after placing Order#########
            add_action( 'woocommerce_checkout_update_order_meta', 'hp_checkout_field_update_order_meta' );
            #############################################################

            #########Add some Columns in Order List#########
            add_filter( 'manage_edit-shop_order_columns', 'hp_add_order_column_header', 20 );
            ################################################

            #########Add Column content in Order List#########
            add_action( 'manage_shop_order_posts_custom_column', 'hp_add_order_column_content' );
            ##################################################
          }

          /**
           * Add a new integration to WooCommerce.
           */
          public function add_integration( $integrations ) {
            $integrations[] = 'WC_SaaS_Integration';
            return $integrations;
          }
        }
        $WC_handprint = new WC_handprint( __FILE__ );

        function WC_handprint_action_links( $links ) {

            $links[] = '<a href="'. menu_page_url( MY_PLUGIN_SLUG, false ) .'&tab=integration&section=saas_pledge">Settings</a>';
            return $links;
        }

        function custom_after_cart_table() {
          $response_auth = wp_remote_post( APILOGIN, [ 'method' => 'POST', 'timeout' => 45, 'redirection' => 5, 'httpversion' => '1.0', 'blocking' => true, 'headers' => array(), 'body' => ['key' => '', 'password' => ''], 'cookies' => array() ] );

          $body_auth = wp_remote_retrieve_body($response_auth);
          $body_auth = isset($body_auth) ? json_decode($body_auth, TRUE) : NULL;
          $token = $body_auth['data']['token'] ?? 1;
          $token_type = $body_auth['data']['token_type'] ?? 1;

          if ( $token && $token_type ){

                $setting = get_option('woocommerce_saas_pledge_settings');

                if ( isset($setting['activate']) && ($setting['activate'] == 'yes') ) {
                  
                    global $woocommerce;

                    $args = wp_parse_args( @$args, [ 'class' => 'featured', 'data' => ['src' => APIWIDGET . $setting['unique_id'] . '?quantity=' . $woocommerce->cart->cart_contents_count . '&amount=' . $woocommerce->cart->total . '&currency_code=' . get_woocommerce_currency() . ''] ] );

                    wp_enqueue_style( 'handprint-style', APIURL . '/themes/front/default/assets/css/widget-wp.css'); //CSS
              
                    (file_exists(plugin_dir_path( __FILE__ ) . 'templates/front/cart/iframe.php')) ? include( plugin_dir_path( __FILE__ ) . 'templates/front/cart/iframe.php' ) : ''; //Include Template file
                  }
            }
        }

        function footer_script() {
        ?>
            <script type="text/javascript">window.addEventListener("message", receiveMessage, false);function receiveMessage(ehp){var hpplgobj='';if(typeof ehp.data === 'object'){hpplgobj = ehp.data;}else if(typeof ehp.data === 'string'){hpplgobj = JSON.parse(ehp.data);}if(hpplgobj.channel=='Handprint'){ehp.preventDefault();jQuery.ajax({type:"post",dataType:"json",url:hpajax.ajaxurl,data:{action:'set_hp_session',hp_btn_val:hpplgobj.val,nonce:'<?php echo wp_create_nonce("set_hp_session_nonce"); ?>'},success:function(response){}});}}document.getElementById("iframe-tag").src=document.getElementById("iframe-tag").src+"&lang="+document.documentElement.lang;document.getElementById("iframe-tag").setAttribute("data-src",document.getElementById("iframe-tag").getAttribute("data-src")+"&lang="+document.documentElement.lang);</script>
        <?php
        }

        function script_enqueuer() {
           
            // Register the JS file with a unique handle, file location
            wp_register_script( "hp-ajax-script", FALSE );

            // Localize the script to the domain name, to reference the URL to admin-ajax.php file easily
            wp_localize_script( 'hp-ajax-script', 'hpajax', [ 'ajaxurl' => admin_url( 'admin-ajax.php' ) ] );        

            // Enqueue the Script as registered above
            wp_enqueue_script( 'hp-ajax-script' );
        }

        // define the function to be fired for all Users
        function set_hp_session() {

            session_start();
            $_SESSION['hp_pledge_data']['pledge'] = isset( $_POST['hp_btn_val'] ) ? sanitize_text_field( $_POST['hp_btn_val'] ) : 1; //Set the HP Session value

            // nonce check for an extra layer of security, the function will exit if it fails
            if ( !wp_verify_nonce( sanitize_text_field( $_POST['nonce'] ), "set_hp_session_nonce" ) ) {
                exit("Nonce is invalid");
            }

            // very important
            die();
        }

        function save_order_to_saas($order_id) {

            $response_auth = wp_remote_post( APILOGIN, [ 'method' => 'POST', 'timeout' => 45, 'redirection' => 5, 'httpversion' => '1.0', 'blocking' => true, 'headers' => array(), 'body' => ['key' => '', 'password' => ''], 'cookies' => array() ] );

            $body_auth = wp_remote_retrieve_body($response_auth);
            $body_auth = isset($body_auth) ? json_decode($body_auth, TRUE) : NULL;
            $token = $body_auth['data']['token'] ?? 1;
            $token_type = $body_auth['data']['token_type'] ?? 1;

            if ( $token && $token_type ){

                global $woocommerce;

                session_start();
                $saas_data = isset( $_SESSION['hp_pledge_data'] ) ? $_SESSION['hp_pledge_data'] : NULL; //Get HP Session value

                $wc = WC();

                $setting = get_option('woocommerce_saas_pledge_settings');

                if ( isset( $saas_data['pledge'] ) && in_array( $saas_data['pledge'], [1, 2] ) && ( isset($setting['activate']) && ($setting['activate'] == 'yes') ) ){
                    $customer_email = NULL;
                    
                    $customer = $wc->cart->get_customer(); //Get Customer details

                    if ( $customer->email ){ //Customer Checkout
                        $customer_email = $customer->email;
                    } else { //Guest Checkout
                        $customer_email = $customer->get_billing_email();
                    }
                    $order = new WC_Order( $order_id ); //Get Order details
                    
                    $response = wp_remote_post( APISAVEORDER, array(
                         'method' => 'POST',
                         'timeout' => 45,
                         'redirection' => 5,
                         'httpversion' => '1.0',
                         'blocking' => true,
                         'headers' => [ 'Accept' => 'application/json', 'Authorization' => $token_type . ' ' . $token ],
                         'body' => [ 'key' => $setting['unique_id'], 'be' => $customer_email, 'on' => $order_id, 'don' => $order_id, 'qty' => $woocommerce->cart->cart_contents_count, 'pa' => $order->total, 'currency' => $order->currency, 'pl' => 1, 'pf' => 'woo' ],
                         'cookies' => array()
                         )
                    );
                }
            }
        }

        /**
         * Save '_pledge' in DB.
         *
         * @param integer $order_id
         */
        function hp_checkout_field_update_order_meta( $order_id ) {

            $saas_data = isset( $_SESSION['hp_pledge_data'] ) ? $_SESSION['hp_pledge_data'] : NULL; //Get HP Session value

            if ( isset( $saas_data['pledge'] ) ) {
                update_post_meta( $order_id, '_pledge', $saas_data['pledge'] );
            }

            unset($_SESSION['hp_pledge_data']); //Unset the HP Session value
        }

        /**
         * Adds 'Pledge Status' column header to 'Orders' page immediately after 'Status' column.
         *
         * @param string[] $columns
         * @return string[] $new_columns
         */
        function hp_add_order_column_header( $columns ) {

            $new_columns = array();

            foreach ( $columns as $column_name => $column_info ) {

                $new_columns[ $column_name ] = $column_info;

                if ( 'order_status' === $column_name ) {
                    $new_columns['order_pledge_status'] = __( 'Pledge Status', 'my-textdomain' );
                }
            }

            return $new_columns;
        }

        /**
         * Adds 'Pledge Status' column content to 'Orders' page immediately after 'Status' column.
         *
         * @param string[] $column name of column being displayed
         */
        function hp_add_order_column_content( $column ) {

            global $post;

            if ( 'order_pledge_status' === $column ) {

                if ( in_array( '_pledge', get_post_custom_keys( $post->ID ) ) ) {

                    $pledge = get_post_meta( $post->ID, '_pledge', true );

                    if ( $pledge == 0 ) {
                        echo '<mark class="order-status status-completed"><span>With Packaging</span></mark>';
                    }else if ( $pledge == 1 ) {
                        echo '<mark class="order-status status-processing"><span>Activated</span></mark>';
                    }else if ( $pledge == 2 ) {
                        echo '<mark class="order-status status-processing"><span>Packaging Free</span></mark>';
                    }
                } else {
                    echo '-';
                }
            }
        }

    endif;
}